;	dual purpose pic
;	can be configured as ps2 keyboard or ir remote receiver
;




;
;	Programme name atkb_new.asm
;
;	DATE: 27/2/2009 - start date
;	D.G. MORTIMER
;	
;
;
;	WRITTEN FOR PIC 16F88
;
;	AT KEYBOARD DECODER - 8 BIT ASCII OUTPUT
;
;	sends data as two bytes viz. ascii key code and function key indicator
;
;	function code 0x00 = normal key
;	function code 0x01 = extended code
;	function code 0x02 = F_key
;		
;	Last modified 22/8/2009
;	Last mod - to include alt and crtl functions
;........................................................................

	#include p16f88.inc		;assembler directives
 	list p=p16f88



	keycount equ h'20'		;used in inkey routine
	kda	equ	1 	;data from keyboard port a
	kcl	equ	0	;clock from keyboard
	scancode equ h'21'		;data read from keyboard
	code1 equ h'22'		;main key code
	code2 equ h'23'		;second scan code
	index equ h'24'		;scan code position
	offset equ h'25'		;lookdown pointer
	pointer equ h'26'		;index for F_key look up

	flags equ h'27'		;flags register
	shift equ 3		;shift flag
	caps_flag equ 2		;bit position of caps lock flag
	alt equ 0		;bit position of alt key flag
	ctrl equ 1		;bit position of ctrl flag
	
	leds equ h'ed'		;keyboard leds
	keyleds equ h'28'		;keyboard leds status
	caps equ 4		;caps lock led value
	ready equ 6		;port a output
	ack equ 7		;port a input from i2c device - handshake	

	
	uSec10 equ h'29'		;delay variables
	ms1 equ h'2a'
	sec1 equ h'2b'
	ms10 equ h'2c'

	delay_byte equ h'2d'
	databyte equ h'2e' 		;register for data to o/p or i/p

	tempw equ h'70'
	status_temp equ h'71'
	fsr_temp equ h'72'
	
	parity equ h'2f'
	bitcount equ h'30'
	temp_data equ h'31'

	__config config1, _lvp_off & _boden_off & _mclr_off & _wdt_off & _intrc_io & _cp_off


	ORG 0		;LOCATION OF RESET VECTOR
	GOTO START	

	ORG  4		;LOCATION OF PERIPHERAL INTERRUPT VECTOR 
	GOTO INTERRUPT	;INTERRUPT ROUTINE LOCATION

	ORG 6		;LOCATION OF RESET START PROGRAMME

start:
	clrf porta		;get rid of any leftover data
	clrf portb
	bcf status,irp
	bcf status,rp1
	bsf status,rp0		;bank 1
	clrf ansel
	bsf trisa,2		;set RA2 as input
	bcf status,rp0		;bank 0
	btfsc porta,2		;check chip mode
	goto start_ir		;go and set up ir receiver mode
	bsf status,rp0		;back to bank 1 - carry on with set up
	clrf trisb		;port b outputs
	movlw b'10111111'
	movwf trisa		;port a inputs
	movlw b'01100000'	;set int rc osc to 4MHz
	movwf osccon
	bcf status,rp0		;bank 0

;===================================================================
;                    end of pic setup routine
;  		beginning of main-loop initialisation
;===================================================================
	
;===================================================================
;       main slave function programme starts here
;===================================================================
;===================================================================
	bcf porta,ready		;tell i2c no data
	movlw 1
	call delaysec		;delay 1 second
	clrf scancode
	clrf portb
	clrf flags
	clrf keyleds
	clrf code1
	clrf code2

main_loop:
	btfss porta,ack		;wait for previous data to be read
	goto main_loop	
	
	call inkey
	movf scancode,w		;check if key released code
	sublw h'f0'
	btfsc status,z
	goto keyrel	
	call check_f_key	;check for F key and set func code
	movf scancode,w
	sublw h'e0'		;check and set extended code byte
	btfss status,z
	goto not_ext		;not function key
	movlw 1
	movwf code2		;set value of function code
	goto main_loop		;get next byte
not_ext:
	movf scancode,w
	sublw h'58'		;check if caps lock key
	btfsc status,z
	goto caps_lock		;set caps lock led

	movf scancode,w		;check for shift left key
	sublw h'12'		;and set up shift flag
	btfsc status,z
	goto set_shift
	movf scancode,w		;check for shift right key
	sublw h'59'		;and set up shift flag
	btfsc status,z	
	goto set_shift

	movf scancode,w		;check for  ctrl key key
	sublw h'14'		;and set up ctrl flag
	btfsc status,z
	goto set_ctrl

	movf scancode,w		;check for alt key
	sublw h'11'		;and set up alt flag
	btfsc status,z
	goto set_alt	
	goto not_modify

set_shift:			;set shift flag
	bsf flags,shift
	goto main_loop
set_ctrl:
	bsf flags,ctrl		;set ctrl flag
	goto main_loop
set_alt:
	bsf flags,alt		;set alt flag
	goto main_loop


not_modify:

	call lookdown		;get the key position index
	decf index,f
	sublw 255		;check if a valid key - no key from lookdown = 255
	btfsc status,z
	goto main_loop
	
	btfsc flags,alt		;check if alt flag set
	goto get_alt_code	;yes, so get key code for alt function

	btfsc flags,ctrl	;check if ctrl flag set
	goto get_ctrl_code	;yes, so get key code for ctrl function
				;not ctrl or alt, so do shift/no shift function.

	btfss flags,shift
	goto no_shift

	incf index,f
	movlw low,uppercase
	addwf index,f
	movlw high,uppercase
	btfsc status,c
	addlw 1
	movwf pclath
	movf index,w
	call uppercase

	movwf temp_data		;save current byte value
	btfss flags,caps_flag	;check if caps lock on
	goto notcaps
	sublw 64		;check if >= "a"
	btfsc status,c
	goto notcapsa
	movlw 91		;check if <= "z"
	subwf temp_data,w
	btfsc status,c
	goto notcapsa
	movf temp_data,w
	xorlw h'20'		;caps lock and shift, so lowercase
	goto notcaps	
	
no_shift:

	incf index,f
	movlw low,lowercase
	addwf index,f
	movlw high,lowercase
	btfsc status,c
	addlw 1
	movwf pclath
	movf index,w

	call lowercase		;get key ascii code
	movwf temp_data		;save current byte value
	btfss flags,caps_flag	;check if caps lock on
	goto notcaps
	sublw 96		;check if >= "a"
	btfsc status,c
	goto notcapsa
	movlw 123		;check if <= "z"
	subwf temp_data,w
	btfsc status,c
	goto notcapsa
	movf temp_data,w
	xorlw h'20'		;caps lock and no shift, so uppercase
	goto notcaps	
notcapsa:
	movf temp_data,w
notcaps:
	goto send_data	

get_alt_code:			;get value of alt key function

	incf index,f
	movlw low,alt_key
	addwf index,f
	movlw high,alt_key
	btfsc status,c
	addlw 1
	movwf pclath
	movf index,w
	call alt_key
	goto send_data		;send to i2c

get_ctrl_code:			;get value of ctrl key function

	incf index,f
	movlw low,control_key
	addwf index,f
	movlw high,control_key
	btfsc status,c
	addlw 1
	movwf pclath
	movf index,w
	call control_key		;send to i2c
	goto send_data

keyrel:
	clrf code2		;clear funtion key flag	
	call inkey

	movf scancode,w
	sublw h'e0'		;check and ignore extended code byte
	btfsc status,z	
	call inkey		;get extra byte for extended code

	movlw h'12'		;check if shift left release
	subwf scancode,w
	btfsc status,z
	bcf flags,shift		;clear shift flag
	movlw h'59'		;check if shift right release
	subwf scancode,w
	btfsc status,z
	bcf flags,shift		;clear shift flag
	movlw h'14'		;check if ctrl key
	subwf scancode,w
	btfsc status,z
	bcf flags,ctrl		;clear ctrl flag
	movlw h'11'		;check if alt key
	subwf scancode,w
	btfsc status,z
	bcf flags,alt		;clear alt flag

	goto main_loop


send_data:
	movwf portb		;send key data in w to i2c device
	bsf porta,ready		;tell i2c data byte is present

wait_ack_lo:
	btfsc porta,ack		;wait for i2c to ask for byte 2
	goto wait_ack_lo	;
	movf code2,w
	movwf portb		;send the function code value
	bcf porta,ready		;indicate  valid data
	goto main_loop	
;==============================================================

	;receive data from keyboard

inkey:
	movlw 8
	movwf keycount		;set 8 bit in
	clrf scancode		;start off with clear slate
waitstart:
	btfsc porta,kcl
	goto waitstart
	call clocklo		;wait for key press
	call clockhi		;bypass start space

inloop:
	call clocklo		;wait for clock to go low
	bcf status,c
	btfsc porta,kda		;check if bit = 1 or 0
	bsf status,c
	rrf scancode,f		;but bit in data byte
	call clockhi
	decfsz keycount,f
	goto inloop
	call clocklo
	call clockhi		;skip parity bit
	call clocklo
	call clockhi		;skip stop bit
	return

clockhi:
	btfss 	porta,kcl	;wait for clock to go high
	goto clockhi		;clock still low
	return
clocklo:
	btfsc porta,kcl		;wait for clock to go low
	goto clocklo		;clock still high
	return

caps_lock:
	movlw caps
	xorwf flags,f		;toggle caps lock flag
	xorwf keyleds,f		;toggle caps lock led
	
	bsf porta,kda
	bsf porta,kcl

	bsf status,rp0
	bcf trisa,kcl		;kcl op
	bcf trisa,kda		;kda op
	bcf status,rp0

	bcf porta,kcl		;clock low

	movlw 12		;delay 120uSec
	call delay10
	
	bcf porta,kda		;kda low

	bsf status,rp0
	bsf trisa,kcl		;kda ip - release to pu
	bcf status,rp0
	
	
	movlw leds		;output command for caps lock
	movwf databyte
	call wrkb8

	call inkey		;wait for 'fa' ack

	bsf porta,kda
	bsf porta,kcl

	bsf status,rp0
	bcf trisa,kcl		;kcl op
	bcf trisa,kda		;kda op
	bcf status,rp0

	bcf porta,kcl		;clock low

	movlw 12		;delay 120uSec
	call delay10
	
	bcf porta,kda		;kda low

	bsf status,rp0
	bsf trisa,kcl		;kda ip - release to pu
	bcf status,rp0

	movf keyleds,w
	movwf databyte
	call wrkb8

	call inkey		;wait for 'fa' ack
	goto main_loop

wrkb8:

	movlw 1
	movwf parity		;preset parity bit
	movlw 8
	movwf bitcount
	call clocklo		;start space
wrkbloop:
	call parcheck		;update parity
	bcf porta,kda		;preclear data
	btfsc databyte,0	;check bit status
	bsf porta,kda		;set data if necessary
	call clockhi		;wait for kb clock
	call clocklo
	rrf databyte,f		;get next data bit in bit 0
	decfsz bitcount,f	;check if 8 bits out
	goto wrkbloop
	bcf porta,kda		;preset parity bit
	movf parity,f		;check state of parity bit
	btfss status,z
	bsf porta,kda
	call clockhi
	call clocklo		;wait for parity clock pulse
	bsf status,rp0
	bsf trisa,kda
	bcf status,rp0		;set data to pull ups
	call clockhi		;stop mark (wait for ack)
	call clocklo
	call clockhi
	return

parcheck:
	movf databyte,w
	andlw 1
	xorwf parity,f
	return





;===================================================================
;delay routines - set for 4MHz clock speed
;===================================================================


delay10:			;10uSec per loop
	movwf usec10		;come in with uSx10 count in w reg
delay10a:
	nop			;pad out to 10uSecs
	nop
	nop
	nop
	nop
	nop
	decfsz usec10,f
	goto delay10a		;loop count 10uSec loops
	return

delayms:
	movwf ms1		;come in with mSec count in W reg
delaymsa:
	movlw 100		;set up for 100 x 10uSec = 1 mSec
	call delay10
	decfsz ms1,f
	goto delaymsa
	return

delayms10:			;come in with mSx10 count in W reg
	movwf ms10
delayms10a:
	movlw 10
	call delayms
	decfsz ms10,f
	goto delayms10a
	return

delaysec:
	movwf sec1		;come in with seconds count in W reg
delayseca:
	movlw 100		;1000 mSec
	call delayms10			
	decfsz sec1,f
	goto delayseca
	return

;===========================================================================
interrupt:			;not used, but just in case
	retfie


;===========================================================================
;	Lookup tables start here
;===========================================================================

lowercase:			;find lower case character in list of AT keys
	movwf pcl		
	retlw	h'61'	;'a'
	retlw	h'62'	;'b'
	retlw	h'63'	;'c'
	retlw	h'64'	;'d'
	retlw	h'65'	;'e'
	retlw	h'66'	;'f'
	retlw	h'67'	;'g'
	retlw	h'68'	;'h'
	retlw	h'69'	;'i'	
	retlw	h'6a'	;'j'
	retlw	h'6b'	;'k'	
	retlw	h'6c'	;'l'
	retlw	h'6d'	;'m'
	retlw	h'6e'	;'n'
	retlw	h'6f'	;'o'
	retlw	h'70'	;'p'
	retlw	h'71'	;'q'
	retlw	h'72'	;'r'
	retlw	h'73'	;'s'
	retlw	h'74'	;'t'
	retlw	h'75'	;'u'
	retlw	h'76'	;'v'
	retlw	h'77'	;'w'	
	retlw	h'78'	;'x'
	retlw	h'79'	;'y'
	retlw	h'7a'	;'z'	
	retlw	h'30'	;'0'
	retlw	h'31'	;'1'
	retlw	h'32'	;'2'
	retlw	h'33'	;'3'
	retlw	h'34'	;'4'
	retlw	h'35'	;'5'
	retlw	h'36'	;'6'
	retlw	h'37'	;'7'
	retlw	h'38'	;'8'
	retlw	h'39'	;'9'
	retlw	h'20'	;'sp'
	retlw	h'2e'	;'.'
	retlw	h'2c'	;','
	retlw	h'2a'	;'*'
	retlw	h'2b'	;'+'	
	retlw	h'0d'	;'enter'
	retlw	h'3d'	;'='
	retlw	h'2f'	;'/'
	retlw	h'00'	;shift l
	retlW	h'00'	;shift r
	retlW	h'30'	;pad 0	47
	retlW	h'31'	;pad 1	49
	retlW	h'32'	;pad 2	50
	retlW	h'33'	;pad 3	51
	retlW	h'34'	;pad 4	52
	retlW	h'35'	;pad 5	53
	retlW	h'36'	;pad 6	54
	retlW	h'37'	;pad 7	55
	retlW	h'38'	;pad 8	56
	retlW	h'39'	;pad 9	57
	retlW	92	;\ |	58
	retlW	h'09'	;tab	59
	retlW	h'00'	;	60
	retlW	h'60'	;`~	61
	retlW	h'1b'	;esc	62
	retlW	h'2f'	;pad /	63
	retlW	h'2a'	;pad *	64
	retlW	h'2d'	;pad -	65
	retlW	h'2b'	;pad +	66
	retlW	h'2e'	;pad .	67
	retlW	h'2d'	;-	68
	retlW	h'27'	;'"	69
	retlW	h'3b'	;;:	70
	retlW	h'5b'	;[{	71
	retlW	93	;]}	72
	retlW	h'3b'	;F1	73
	retlW	h'3c'	;F2	74
	retlW	h'3d'	;F3	75
	retlW	h'3e'	;F4	76
	retlW	h'3f'	;F5	77
	retlW	h'40'	;F6	78
	retlW	h'41'	;F7	79
	retlW	h'42'	;F8	80
	retlW	h'43'	;F9	81
	retlW	h'44'	;F10	82
	retlW	h'85'	;F11	83
	retlW	h'86'	;F12	84
	retlw	h'00'	;null chacter = end of table
	

uppercase:		;uppercase values of keys
	
	movwf pcl
	retlw	h'41'	;'a'
	retlw	h'42'	;'b'
	retlw	h'43'	;'c'
	retlw	h'44'	;'d'
	retlw	h'45'	;'e'
	retlw	h'46'	;'f'
	retlw	h'47'	;'g'
	retlw	h'48'	;'h'
	retlw	h'49'	;'i'	
	retlw	h'4a'	;'j'
	retlw	h'4b'	;'k'	
	retlw	h'4c'	;'l'
	retlw	h'4d'	;'m'
	retlw	h'4e'	;'n'
	retlw	h'4f'	;'o'
	retlw	h'50'	;'p'
	retlw	h'51'	;'q'
	retlw	h'52'	;'r'
	retlw	h'53'	;'s'
	retlw	h'54'	;'t'
	retlw	h'55'	;'u'
	retlw	h'56'	;'v'
	retlw	h'57'	;'w'	
	retlw	h'58'	;'x'
	retlw	h'59'	;'y'
	retlw	h'5a'	;'z'	
	retlw	h'29'	;'0'
	retlw	h'21'	;'1'
	retlw	h'40'	;'2'
	retlw	h'23'	;'3'
	retlw	h'24'	;'4'
	retlw	h'25'	;'5'
	retlw	h'5e'	;'6'
	retlw	h'26'	;'7'
	retlw	h'2a'	;'8'
	retlw	h'28'	;'9'
	retlw	h'20'	;'sp'
	retlw	h'3e'	;'.'
	retlw	h'3c'	;','
	retlw	h'2a'	;'*'
	retlw	h'2b'	;'+'	
	retlw	h'0d'	;'enter'
	retlw	h'2b'	;'='
	retlw	h'3f'	;'/'
	retlw	h'00'	;shift l
	retlW	h'00'	;shift r
	retlW	h'30'	;pad 0	47
	retlW	h'31'	;pad 1	49
	retlW	h'32'	;pad 2	50
	retlW	h'33'	;pad 3	51
	retlW	h'34'	;pad 4	52
	retlW	h'35'	;pad 5	53
	retlW	h'36'	;pad 6	54
	retlW	h'37'	;pad 7	55
	retlW	h'38'	;pad 8	56
	retlW	h'39'	;pad 9	57
	retlW	h'7c'	;\ |	58
	retlW	h'0f'	;tab	59
	retlW	h'00'	;	60
	retlW	h'7e'	;`~	61
	retlW	h'1b'	;esc	62
	retlW	h'2f'	;pad /	63
	retlW	h'2a'	;pad *	64
	retlW	h'2d'	;pad -	65
	retlW	h'2b'	;pad +	66
	retlW	h'2e'	;pad .	67
	retlW	h'5f'	;-	68
	retlW	h'22'	;'"	69
	retlW	h'3a'	;;:	70
	retlW	h'7b'	;[{	71
	retlW	h'7d'	;]}	72
	retlW	h'05'	;F1	73
	retlW	h'06'	;F2	74
	retlW	h'04'	;F3	75
	retlW	h'0c'	;F4	76
	retlW	h'03'	;F5	77
	retlW	h'0b'	;F6	78
	retlW	h'83'	;F7	79
	retlW	h'0a'	;F8	80
	retlW	h'01'	;F9	81
	retlW	h'09'	;F10	82
	retlW	h'78'	;F11	83
	retlW	h'07'	;F12	84
	retlw	h'00'	;null chacter = end of table

	
	
control_key:			;ctrl key value of key pressed		
	movwf 	PCL		;set table offset

	retlw	h'01'	;'a'
	retlw	h'02'	;'b'
	retlw	h'03'	;'c'
	retlw	h'04'	;'d'
	retlw	h'05'	;'e'
	retlw	h'06'	;'f'
	retlw	h'07'	;'g'
	retlw	h'08'	;'h'
	retlw	h'09'	;'i'	
	retlw	h'0a'	;'j'
	retlw	h'0b'	;'k'	
	retlw	h'0c'	;'l'
	retlw	h'0d'	;'m'
	retlw	h'0e'	;'n'
	retlw	h'0f'	;'o'
	retlw	h'10'	;'p'
	retlw	h'11'	;'q'
	retlw	h'12'	;'r'
	retlw	h'13'	;'s'
	retlw	h'14'	;'t'
	retlw	h'15'	;'u'
	retlw	h'16'	;'v'
	retlw	h'17'	;'w'	
	retlw	h'18'	;'x'
	retlw	h'19'	;'y'
	retlw	h'1a'	;'z'	
	retlw	h'1b'	;'0'
	retlw	h'ff'	;'1'
	retlw	h'ff'	;'2'
	retlw	h'ff'	;'3'
	retlw	h'ff'	;'4'
	retlw	h'ff'	;'5'
	retlw	h'1e'	;'6'
	retlw	h'ff'	;'7'
	retlw	h'ff'	;'8'
	retlw	h'ff'	;'9'
	retlw	h'ff'	;'sp'
	retlw	h'1f'	;'.'
	retlw	h'2c'	;','
	retlw	h'2a'	;'*'
	retlw	h'2b'	;'+'	
	retlw	h'0d'	;'enter'
	retlw	h'3d'	;'='
	retlw	h'2f'	;'/'
	retlw	h'00'	;shift l
	retlW	h'00'	;shift r
	retlW	h'30'	;pad 0	47
	retlW	h'31'	;pad 1	49
	retlW	h'32'	;pad 2	50
	retlW	h'33'	;pad 3	51
	retlW	h'34'	;pad 4	52
	retlW	h'35'	;pad 5	53
	retlW	h'36'	;pad 6	54
	retlW	h'37'	;pad 7	55
	retlW	h'38'	;pad 8	56
	retlW	h'39'	;pad 9	57
	retlW	h'1c'	;\ |	58
	retlW	h'0f'	;tab	59
	retlW	h'00'	;	60
	retlW	h'60'	;`~	61
	retlW	h'1b'	;esc	62
	retlW	h'2f'	;pad /	63
	retlW	h'2a'	;pad *	64
	retlW	h'2d'	;pad -	65
	retlW	h'2b'	;pad +	66
	retlW	h'2e'	;pad .	67
	retlW	h'2d'	;-	68
	retlW	h'00'	;'"	69
	retlW	h'00'	;;:	70
	retlW	h'52'	;[{	71
	retlW	h'4c'	;]}	72
	retlW	h'05'	;F1	73
	retlW	h'06'	;F2	74
	retlW	h'04'	;F3	75
	retlW	h'0c'	;F4	76
	retlW	h'03'	;F5	77
	retlW	h'0b'	;F6	78
	retlW	h'83'	;F7	79
	retlW	h'0a'	;F8	80
	retlW	h'01'	;F9	81
	retlW	h'09'	;F10	82
	retlW	h'78'	;F11	83
	retlW	h'07'	;F12	84
	retlw	h'00'	;null chacter = end of table

alt_key:				;get alt key value of key pressed
	movwf 	PCL	;set table offset

	retlw	30	;'a'
	retlw	48	;'b'
	retlw	46	;'c'
	retlw	32	;'d'
	retlw	18	;'e'
	retlw	33	;'f'
	retlw	34	;'g'
	retlw	35	;'h'
	retlw	23	;'i'	
	retlw	36	;'j'
	retlw	37	;'k'	
	retlw	38	;'l'
	retlw	50	;'m'
	retlw	49	;'n'
	retlw	24	;'o'
	retlw	25	;'p'
	retlw	16	;'q'
	retlw	19	;'r'
	retlw	31	;'s'
	retlw	20	;'t'
	retlw	22	;'u'
	retlw	47	;'v'
	retlw	17	;'w'	
	retlw	45	;'x'
	retlw	21	;'y'
	retlw	44	;'z'	
	retlw	129	;'0'
	retlw	120	;'1'
	retlw	121	;'2'
	retlw	122	;'3'
	retlw	123	;'4'
	retlw	124	;'5'
	retlw	125	;'6'
	retlw	126	;'7'
	retlw	126	;'8'
	retlw	127	;'9'
	retlw	32	;'sp'
	retlw	52	;'.'
	retlw	51	;','
	retlw	0	;'*'
	retlw	55	;'+'	
	retlw	28	;'enter'
	retlw	131	;'='
	retlw	53	;'/'
	retlw	0	;shift l
	retlW	h'00'	;shift r
	retlW	129	;pad 0	47
	retlW	120	;pad 1	49
	retlW	121	;pad 2	50
	retlW	122	;pad 3	51
	retlW	123	;pad 4	52
	retlW	124	;pad 5	53
	retlW	125	;pad 6	54
	retlW	126	;pad 7	55
	retlW	126	;pad 8	56
	retlW	127	;pad 9	57
	retlW	43	;\ |	58
	retlW	165	;tab	59
	retlW	h'00'	;	60
	retlW	h'60'	;`~	61
	retlW	h'1b'	;esc	62
	retlW	53	;pad /	63
	retlW	0	;pad *	64
	retlW	164	;pad -	65
	retlW	55	;pad +	66
	retlW	h'2e'	;pad .	67
	retlW	130	;-	68
	retlW	40	;'"	69
	retlW	39	;;:	70
	retlW	h'52'	;[{	71
	retlW	h'4c'	;]}	72
	retlW	h'05'	;F1	73
	retlW	h'06'	;F2	74
	retlW	h'04'	;F3	75
	retlW	h'0c'	;F4	76
	retlW	h'03'	;F5	77
	retlW	h'0b'	;F6	78
	retlW	h'83'	;F7	79
	retlW	h'0a'	;F8	80
	retlW	h'01'	;F9	81
	retlW	h'09'	;F10	82
	retlW	h'78'	;F11	83
	retlW	h'07'	;F12	84

	retlw	h'00'	;null chacter = end of table


LOOKdown:
	
	clrf	index		;point to start of table

lookdownloop:
	incf	index,f
	movf index,w
	movwf offset
	movlw low,scan_code_table
	addwf offset,f
	movlw high,scan_code_table
	btfsc status,c
	addlw 1
	movwf pclath

	movf	offset,w		;load value of index in w reg
	call	scan_code_table	;go and get value of literal at table+index location
	movwf 	code1
	subwf	scancode,w	;check if match
	btfsc	status,z	
	return			;back to caller with location in index
	movf	code1,f
	btfss	status,z
	goto	lookdownloop
	retlw 255		;indicate no match
	


scan_code_table:				;find character in list of AT scan codes
	movwf 	PCL	;set table offset

	retlw	h'1c'	;'a'	1
	retlw	h'32'	;'b'	2
	retlw	h'21'	;'c'	3
	retlw	h'23'	;'d'	4
	retlw	h'24'	;'e'	5
	retlw	h'2b'	;'f'	6
	retlw	h'34'	;'g'	7
	retlw	h'33'	;'h'	8
	retlw	h'43'	;'i'	9
	retlw	h'3b'	;'j'	10
	retlw	h'42'	;'k'	11	
	retlw	h'4b'	;'l'	12
	retlw	h'3a'	;'m'	13
	retlw	h'31'	;'n'	14
	retlw	h'44'	;'o'	15
	retlw	h'4d'	;'p'	16
	retlw	h'15'	;'q'	17
	retlw	h'2d'	;'r'	18
	retlw	h'1b'	;'s'	19
	retlw	h'2c'	;'t'	20
	retlw	h'3c'	;'u'	21
	retlw	h'2a'	;'v'	22
	retlw	h'1d'	;'w'	23
	retlw	h'22'	;'x'	24
	retlw	h'35'	;'y'	25
	retlw	h'1a'	;'z'	26
	retlw	h'45'	;'0'	27
	retlw	h'16'	;'1'	28
	retlw	h'1e'	;'2'	29
	retlw	h'26'	;'3'	30
	retlw	h'25'	;'4'	31
	retlw	h'2e'	;'5'	32
	retlw	h'36'	;'6'	33
	retlw	h'3d'	;'7'	34
	retlw	h'3e'	;'8'	35
	retlw	h'46'	;'9'	36
	retlw	h'29'	;'sp'	37
	retlw	h'49'	;'.'	38
	retlw	h'41'	;','	39
	retlw	h'7c'	;'*'	40
	retlw	h'79'	;'+'	41	
	retlw	h'5a'	;'enter'42
	retlw	h'55'	;'='	43
	retlw	h'4a'	;'/'	44
	retlW	h'12'	;shiftL	45
	retlW	h'59'	;shiftr	46
	retlW	h'70'	;pad 0	47
	retlW	h'69'	;pad 1	49
	retlW	h'72'	;pad 2	50
	retlW	h'7a'	;pad 3	51
	retlW	h'6b'	;pad 4	52
	retlW	h'73'	;pad 5	53
	retlW	h'74'	;pad 6	54
	retlW	h'6c'	;pad 7	55
	retlW	h'75'	;pad 8	56
	retlW	h'7d'	;pad 9	57
	retlW	h'5d'	;\ |	58
	retlW	h'0d'	;tab	59
	retlW	h'ff'	;	60  spare
	retlW	h'0e'	;`~	61
	retlW	h'76'	;esc	62
	retlW	h'4a'	;pad /	63
	retlW	h'7c'	;pad *	64
	retlW	h'7b'	;pad -	65
	retlW	h'79'	;pad +	66
	retlW	h'71'	;pad .	67
	retlW	h'4e'	;-	68
	retlW	h'52'	;'"	69
	retlW	h'4c'	;;:	70
	retlW	h'54'	;[{	71
	retlW	h'5b'	;]}	72
	retlW	h'05'	;F1	73
	retlW	h'06'	;F2	74
	retlW	h'04'	;F3	75
	retlW	h'0c'	;F4	76
	retlW	h'03'	;F5	77
	retlW	h'0b'	;F6	78
	retlW	h'83'	;F7	79
	retlW	h'0a'	;F8	80
	retlW	h'01'	;F9	81
	retlW	h'09'	;F10	82
	retlW	h'78'	;F11	83
	retlW	h'07'	;F12	84
	retlw	h'00'	;null chacter = end of table

check_f_key:

	clrf	pointer		;point to start of table

f_key_loop:
	incf	pointer,f
	movf pointer,w
	movwf offset
	movlw low,f_key_table
	addwf offset,f
	movlw high,f_key_table
	btfsc status,c
	addlw 1
	movwf pclath			
	movf	offset,w	;load value of index in w reg
	call	f_key_table	;go and get value of literal at table+pointer location
	movwf	code1
	subwf	scancode,w	;check if match
	btfsc	status,z	
	goto 	set_F_flag	;
	movf 	code1,f		;check if end of table
	btfss	status,z	
	goto	f_key_loop
	return
set_f_flag:
	movlw 2			;indicate a F key
	movwf code2
	return	

f_key_table:			;get key in F1 to F12 table
	movwf 	PCL	;set table offset
	retlw	h'05'	;'a'	F1
	retlw	h'06'	;'b'	F2
	retlw	h'04'	;'c'	F3
	retlw	h'0c'	;'d'	F4
	retlw	h'03'	;'e'	F5
	retlw	h'0b'	;'f'	F6
	retlw	h'83'	;'g'	F7
	retlw	h'0a'	;'h'	F8
	retlw	h'01'	;'i'	F9
	retlw	h'09'	;'j'	F10
	retlw	h'78'	;'k'	F11	
	retlw	h'07'	;'l'	F12
	retlw   h'00'	;null = end of table

;=====================================================================
; 	end of ps2 keyboard
;	start of ir remote receiver
;=====================================================================   

;===================================================================
;	Programme written by D G Mortimer
;
;	Programme name:ir_new.asm
;
;	Written for pic16f88
;
;
;
;
; 	programme to read code from infra red remote
;
;	and send data via 2 line handshake to i2c slave.
;
;	Sends two bytes of data viz. button code and device code
;	first byte: button code
;	second byte: device code (dvd, stb,etc)
;
;	Last modified 24/8/2009 - adapted for new duo board
;	reads data independently of i2c activity
;
;===================================================================
;
	


	input equ 0		;port a bit 0 - ir input pin
	width res 1		;pulse width count
	byte0 res 1
	byte1 res 1
	byte2 res 1
	byte3 res 1
	addr0 res 1		;device address byte
	addr1 res 1		;code address byte



;===================================================================
;Main pic set up routine
;===================================================================

start_ir:
	clrf porta		;get rid of any leftover data
	clrf portb
	bcf status,irp
	bcf status,rp1
	bsf status,rp0		;bank 1
	clrf ansel
	clrf trisb		;port b outputs
	movlw b'10111111'
	movwf trisa		;port a inputs
	movlw b'01100000'	;set int rc osc to 4MHz
	movwf osccon
	bcf status,rp0		;bank 0

;===================================================================
;                    end of pic setup routine
;  		beginning of main-loop initialisation
;===================================================================

;===================================================================
;===================================================================
;       main slave function programme starts here
;===================================================================
;===================================================================

	movlw 2
	call delaysec
	clrf portb
	
;===================================================================

begin:
	clrf byte0
	clrf byte1
	clrf byte2
	clrf byte3
	clrf portb
	bcf porta,ready		;indicate to i2c that there is no data yet	

ir_main:
	btfss porta,ack		;wait for previous data to be read
	goto ir_main
	btfsc porta,input	;check for IR data
	goto ir_main		;no start
	movlw 3
	call delayms
	btfsc porta,input
	goto ir_main		;false start
	
ml1:
	btfss porta,input	;wait for input high
	goto ml1
	call pulse_width	;wait for start pulse lo to hi
ml2:
	call pulse_width
	btfsc status,z
	goto check_data
	movlw 13		;if width > 1.2mSec then bit is a logic 1
	subwf width,w
	btfss status,c		;carry is clear if a logic 0
	goto set1
set0:
	bcf status,c
	rlf byte0,f
	rlf byte1,f
	rlf byte2,f
	rlf byte3,f
	goto ml2

set1:
	bsf status,c
	rlf byte0,f
	rlf byte1,f
	rlf byte2,f
	rlf byte3,f
	goto ml2

pulse_width:
	clrf width
pw1:	
	movlw 10
	call delay10
	incf width,f 
	movf width,f
	btfsc status,z
	return
	btfsc porta,input	;read input pin
	goto pw1
pw2:
	movlw 10
	call delay10
	incf width,f
	movf width,f
	btfsc status,z
	return
	btfss porta,input
	goto pw2
	return

check_data:
	incfsz byte3,w
	goto ir_main		;invalid data
	comf byte1,w		;check if data complement
	subwf byte0,w
	btfss status,z
	goto ir_main		;invalid data	

send_ir_data:
	movf byte1,w		;send first byte to i2c - key code
	movwf portb
	bsf porta,ready		;tell i2c data byte is present

ir_ack_lo:
	btfsc porta,ack		;wait for i2c to ask for byte 2
	goto ir_ack_lo
	movf byte2,w		;device code
	movwf portb		;send byte 2
	bcf porta,ready		;indicate  valid data

	clrf byte0
	clrf byte1
	clrf byte2
	clrf byte3	 
	goto ir_main	

;===================================================================
	
	
	end			;mandatory instruction 